"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dgeni_1 = require("dgeni");
const mockPackage = require('../mocks/mockPackage');
const path = require('canonical-path');
describe('linkInheritedDocs', () => {
    let dgeni;
    let injector;
    let tsProcessor;
    let linkProcessor;
    beforeEach(() => {
        dgeni = new dgeni_1.Dgeni([mockPackage()]);
        injector = dgeni.configureInjector();
        tsProcessor = injector.get('readTypeScriptModules');
        linkProcessor = injector.get('linkInheritedDocs');
        tsProcessor.basePath = path.resolve(__dirname, '../mocks/linkInheritedDocs');
        tsProcessor.sourceFiles = ['index.ts', 'deps.ts'];
    });
    it('should properly link the inherited docs', () => {
        const docsArray = [];
        tsProcessor.$process(docsArray);
        linkProcessor.$process(docsArray);
        const childDoc = docsArray.find(doc => doc.name === 'Child');
        const firstParentDoc = docsArray.find(doc => doc.name === 'FirstParent');
        const lastParentDoc = docsArray.find(doc => doc.name === 'LastParent');
        expect(childDoc.extendsClauses.map(clause => clause.doc)).toEqual([firstParentDoc]);
        expect(firstParentDoc.extendsClauses.map(clause => clause.doc)).toEqual([lastParentDoc]);
        expect(lastParentDoc.extendsClauses.map(clause => clause.doc)).toEqual([]);
        expect(lastParentDoc.descendants).toEqual([firstParentDoc]);
        expect(firstParentDoc.descendants).toEqual([childDoc]);
        expect(childDoc.descendants).toEqual([]);
    });
    it('should properly resolve members in inherited docs', () => {
        const docsArray = [];
        tsProcessor.$process(docsArray);
        linkProcessor.$process(docsArray);
        const childDoc = docsArray.find(doc => doc.name === 'Child');
        const memberNames = getInheritedMembers(childDoc).map(member => member.name);
        expect(memberNames.length).toBe(3);
        expect(memberNames).toContain('childProp');
        expect(memberNames).toContain('firstParentProp');
        expect(memberNames).toContain('lastParentProp');
    });
    /** Returns a list of all inherited members of a doc. */
    function getInheritedMembers(doc) {
        let members = doc.members || [];
        doc.extendsClauses.forEach(clause => {
            members = members.concat(getInheritedMembers(clause.doc));
        });
        return members;
    }
});
//# sourceMappingURL=linkInheritedDocs.spec.js.map