"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @dgProcessor
 *
 * @description
 * Merge the description from `@param` tags into the parameter docs
 * extracted from the TypeScript
 */
function mergeParameterInfo() {
    return new MergeParameterInfoProcessor();
}
exports.mergeParameterInfo = mergeParameterInfo;
class MergeParameterInfoProcessor {
    constructor() {
        this.$runAfter = ['readTypeScriptModules', 'tags-extracted'];
        this.$runBefore = ['extra-docs-added'];
    }
    $process(docs) {
        docs.forEach((doc) => {
            if (doc.docType === 'parameter') {
                // The `params` property comes from parsing the `@param` jsdoc tags on the container doc
                const paramTag = doc.container.params && doc.container.params.find((param) => param.name === doc.name);
                if (paramTag && paramTag.description) {
                    doc.description = paramTag.description;
                }
            }
        });
    }
}
exports.MergeParameterInfoProcessor = MergeParameterInfoProcessor;
//# sourceMappingURL=mergeParameterInfo.js.map