"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
const ClassExportDoc_1 = require("../api-doc-types/ClassExportDoc");
const ClassLikeExportDoc_1 = require("../api-doc-types/ClassLikeExportDoc");
const convertPrivateClassesToInterfaces_1 = require("./convertPrivateClassesToInterfaces");
const host_1 = require("./ts-host/host");
const FileInfo_1 = require("./TsParser/FileInfo");
describe('convertPrivateClassesToInterfaces', () => {
    const basePath = 'a/b/c';
    const moduleDoc = { id: 'someModule', basePath };
    const mockDeclaration = { getSourceFile: () => typescript_1.createSourceFile('x/y/z', 'blah blah', typescript_1.ScriptTarget.ES5), pos: 0, end: 0 };
    const classSymbol = {
        getDeclarations: () => [mockDeclaration],
        name: 'privateClass',
        valueDeclaration: mockDeclaration,
    };
    let classDoc;
    let docs;
    beforeEach(() => {
        spyOn(FileInfo_1.FileInfo.prototype, 'getRealFilePath').and.callFake((filePath) => filePath);
        classDoc = new ClassExportDoc_1.ClassExportDoc(new host_1.Host(), moduleDoc, classSymbol);
        classDoc.constructorDoc = { internal: true };
        docs = [classDoc];
    });
    it('should convert private class docs to interface docs', () => {
        convertPrivateClassesToInterfaces_1.convertPrivateClassesToInterfaces(docs, false);
        expect(docs[0].docType).toEqual('interface');
    });
    it('should not touch non-private class docs', () => {
        classDoc.constructorDoc = {};
        convertPrivateClassesToInterfaces_1.convertPrivateClassesToInterfaces(docs, false);
        expect(docs[0].docType).toEqual('class');
    });
    it('should convert the heritage since interfaces use `extends` not `implements`', () => {
        const heritage = new ClassLikeExportDoc_1.HeritageInfo({}, '');
        classDoc.implementsClauses = [heritage];
        convertPrivateClassesToInterfaces_1.convertPrivateClassesToInterfaces(docs, false);
        expect(docs[0].extendsClauses).toEqual([heritage]);
    });
    it('should add new injectable reference types, if specified, to the passed in collection', () => {
        convertPrivateClassesToInterfaces_1.convertPrivateClassesToInterfaces(docs, true);
        expect(docs[1]).toEqual(jasmine.objectContaining({
            docType: 'const',
            id: 'someModule/privateClass',
            name: 'privateClass',
            type: 'InjectableReference',
        }));
    });
});
//# sourceMappingURL=convertPrivateClassesToInterfaces.spec.js.map